module.exports = {
  name: 'builtin:dev',
  apply(api) {
    api.cli.command('dev', 'Run app in dev mode', () => {
      const webpackConfig = api.createWebpackConfig()
      const compiler = require('webpack')(webpackConfig)

      const devServerOptions = Object.assign(
        {
          hot: true,
          quiet: true,
          historyApiFallback: true,
          overlay: true,
          disableHostCheck: true,
          publicPath: webpackConfig.output.publicPath
        },
        api.config.devServer
      )

      const existingBefore = devServerOptions.before
      devServerOptions.before = server => {
        server.use(
          require('poi-dev-utils/launchEditorEndpoint'),
          require('launch-editor-middleware')(() =>
            console.log(
              `To specify an editor, sepcify the EDITOR env variable or ` +
                `add "editor" field to your Vue project config.\n`
            )
          )
        )
        existingBefore && existingBefore(server)
      }

      const exitingAfter = devServerOptions.after
      devServerOptions.after = server => {
        exitingAfter && exitingAfter(server)
        api.hooks.invoke('configureDevServer', server)
      }

      const WebpackDevServer = require('webpack-dev-server')
      const server = new WebpackDevServer(compiler, devServerOptions)

      const { host, port } = api.config.devServer
      server.listen(port, host)
    })
  }
}
